#!/usr/bin/env nextflow

nextflow.enable.dsl=2
include { consolidateAnnotation;trainEuGene;buildEGNRepeatFile;launchAllEGN } from './modules/subworkflow/eugene.nf'
include { speciesSpecificRepeats;searchRepeat } from './modules/subworkflow/repeats.nf'
include { seqArbo;buildLargeOverlappingSequences;buildOverlappingSequences;processAdditionalFiles;estPredictions;proteinPredictions;abInitioPredictions;genomeIndex } from './modules/subworkflow/prepare.nf'
include { softwares_info } from './modules/subworkflow/softwares_info.nf'
include {LIPMUTILS_unmaskSoft} from './modules/software/lipmutils/main.nf'

workflow {
	main:

		if (params.help) {
			def help_message = NFcoreTemplate.help(workflow, params, log)
			log.info help_message
			exit 0
		}

		if(! params.output_prefix) { exit 1, 'output_prefix not specified' }
		if(! params.locus_tag_prefix) { exit 1, 'locus_tag_prefix not specified' }
		
		Utils.info("Reading configuration files", params.verbose)
		(params,configMap) = Utils.readParams(params);//,params.cfg) 
		Utils.info("Create config directory ${params.outdir}/01_cfg", params.verbose)
		File cfgDir = new File(params.outdir + "/01_cfg")
		cfgDir.mkdir()
		egn_params_only = Utils.dumpParams("${params.outdir}/01_cfg/run.parameters.cfg",params)
		run_parameters = Channel.fromPath(egn_params_only)
	
		if (params.sequences) { 
			completeGenome = LIPMUTILS_unmaskSoft(Channel.fromPath(params.sequences))
			
			Utils.checkSequenceIds(params.sequences)
			
			Utils.info("Computing genome mode (normal or big)", params.verbose)
			configMap['genome_mode'] = Utils.getGenomeMode(params.sequences)
			Utils.info("Genome mode: " + configMap['genome_mode'], params.verbose)
			
			Utils.info("Computing genome hash", params.verbose)
			configMap['genome_hash'] = Utils.getFastaHash(params.sequences)
			Utils.info("Genome hash:" + configMap['genome_hash'], params.verbose)
			
			if (configMap['genome_size'] > 2**32)
			{
				Utils.info("Genome mode: big")
			}
			params['genome_size'] = configMap['genome_size']
			params['genome_hash'] = configMap['genome_hash']
		} else { exit 1, 'Input genome file not specified!' }
		
		if (params.mode == 'test') {
			//run only some process here
			pseudomolecules = seqArbo(completeGenome)
			overlappingSeq = buildOverlappingSequences(completeGenome)
			proteinEvidences = proteinPredictions(configMap,overlappingSeq)
			//(abInitioEvidences, maskingEvidences,ncRnaEvidences) = abInitioPredictions(completeGenome,pseudomolecules,overlappingSeq)
		}
		else if (params.mode == 'abinitio') {
			//run only some process here
			pseudomolecules = seqArbo(completeGenome)
			overlappingSeq = buildOverlappingSequences(completeGenome)
			largeOverlappingSeq = buildLargeOverlappingSequences(completeGenome)
			(abInitioEvidences, maskingEvidences,ncRnaEvidences) = abInitioPredictions(completeGenome,pseudomolecules,overlappingSeq,largeOverlappingSeq)
			//(abInitioEvidences, maskingEvidences,ncRnaEvidences) = abInitioPredictions(completeGenome,pseudomolecules,overlappingSeq)
		}
		else if (params.mode == 'protein') {
			//run only some process here
			overlappingSeq = buildOverlappingSequences(completeGenome)
			proteinEvidences = proteinPredictions(configMap,overlappingSeq)
			//(abInitioEvidences, maskingEvidences,ncRnaEvidences) = abInitioPredictions(completeGenome,pseudomolecules,overlappingSeq)
		}
		else if (params.mode == 'est') {
			//run only some process here
			genomeIndexPath = genomeIndex(completeGenome,configMap.genome_mode,configMap.genome_hash)
			(estEvidences,rawGmap4Training,allEstConf,trainingSplitEstDBch) = estPredictions(configMap,configMap.genome_mode,configMap.genome_hash,completeGenome,genomeIndexPath)
		}
		else if (params.mode == 'train') {
			genomeIndexPath = genomeIndex(completeGenome,configMap.genome_mode,configMap.genome_hash)
			(estEvidences,rawGmap4Training,allEstConf,trainingSplitEstDBch) = estPredictions(configMap,configMap.genome_mode,configMap.genome_hash,completeGenome,genomeIndexPath)
			(matrix,noncan_cfg) = trainEuGene(completeGenome,configMap,configMap.genome_mode,configMap.genome_hash,rawGmap4Training,genomeIndexPath,trainingSplitEstDBch)
		}
		else if (params.mode == 'repeats') {
			pseudomolecules = seqArbo(completeGenome)
			overlappingSeq = buildOverlappingSequences(completeGenome)
			largeOverlappingSeq = buildLargeOverlappingSequences(completeGenome)
			(abInitioEvidences, maskingEvidences,ncRnaEvidences) = abInitioPredictions(completeGenome,pseudomolecules,overlappingSeq,largeOverlappingSeq)
			genomeIndexPath = genomeIndex(completeGenome,configMap.genome_mode,configMap.genome_hash)
			(estEvidences,rawGmap4Training,allEstConf,trainingSplitEstDBch) = estPredictions(configMap,configMap.genome_mode,configMap.genome_hash,completeGenome,genomeIndexPath)
			proteinEvidences = proteinPredictions(configMap,overlappingSeq)
			
			allAvailableEvidences = abInitioEvidences.concat(estEvidences,proteinEvidences)
			
			allEvidences = allAvailableEvidences.groupTuple().map{ t -> tuple(t.get(0),t.get(1).flatten())}
			(matrix,noncan_cfg) = trainEuGene(completeGenome,configMap,configMap.genome_mode,configMap.genome_hash,rawGmap4Training,genomeIndexPath,trainingSplitEstDBch)
			(repeatFasta, repeatFastaIndex) = speciesSpecificRepeats(completeGenome,pseudomolecules,configMap,matrix,noncan_cfg,allEvidences,allEstConf,run_parameters)
			repeatEvidences = searchRepeat(completeGenome, pseudomolecules,maskingEvidences,repeatFasta, repeatFastaIndex)
			okrepeatfile4egnTuples = buildEGNRepeatFile(configMap,pseudomolecules,repeatEvidences,ncRnaEvidences,estEvidences,proteinEvidences)
			
		}
		else {
			softwares_info()
			pseudomolecules = seqArbo(completeGenome)
			overlappingSeq = buildOverlappingSequences(completeGenome)
			largeOverlappingSeq = buildLargeOverlappingSequences(completeGenome)
			(abInitioEvidences, maskingEvidences,ncRnaEvidences) = abInitioPredictions(completeGenome,pseudomolecules,overlappingSeq,largeOverlappingSeq)
			genomeIndexPath = genomeIndex(completeGenome,configMap.genome_mode,configMap.genome_hash)
			(estEvidences,rawGmap4Training,allEstConf,trainingSplitEstDBch) = estPredictions(configMap,configMap.genome_mode,configMap.genome_hash,completeGenome,genomeIndexPath)
			proteinEvidences = proteinPredictions(configMap,overlappingSeq)
			
			allAvailableEvidences = abInitioEvidences.concat(estEvidences,proteinEvidences)
			if (configMap.additional_list.size() > 0) {
				additionalEvidences = processAdditionalFiles(completeGenome,configMap)
				allAvailableEvidences = allAvailableEvidences.concat(additionalEvidences)
			}

			allEvidences = allAvailableEvidences.groupTuple().map{ t -> tuple(t.get(0),t.get(1).flatten())}
			(matrix,noncan_cfg) = trainEuGene(completeGenome,configMap,configMap.genome_mode,configMap.genome_hash,rawGmap4Training,genomeIndexPath,trainingSplitEstDBch)
			(repeatFasta, repeatFastaIndex) = speciesSpecificRepeats(completeGenome,pseudomolecules,configMap,matrix,noncan_cfg,allEvidences,allEstConf,run_parameters)
			repeatEvidences = searchRepeat(completeGenome, pseudomolecules,maskingEvidences,repeatFasta, repeatFastaIndex)
			okrepeatfile4egnTuples = buildEGNRepeatFile(configMap,pseudomolecules,repeatEvidences,ncRnaEvidences,estEvidences,proteinEvidences)
			
			finalEvidences = ncRnaEvidences.concat(estEvidences,proteinEvidences,repeatEvidences,okrepeatfile4egnTuples)
								.groupTuple().map{ 
											t -> tuple(t.get(0),t.get(1).flatten())
										}
			annotationV1gff3Tuples = launchAllEGN(pseudomolecules,matrix,noncan_cfg,finalEvidences,allEstConf,run_parameters,configMap.genome_mode)
			consolidateAnnotation(completeGenome, annotationV1gff3Tuples, ncRnaEvidences)
		}

		workflow.onComplete {
    		Utils.info("Pipeline completed.\nResults are located in ${params.outdir}", true)
    //println "Execution status: ${ workflow.success ? 'OK' : 'failed' }"
		}
		workflow.onError {
    		Utils.error("Oops... Pipeline execution stopped") //": ${workflow.errorMessage}"
		}
		
}

