#!/bin/bash

ARGS="$*"
CONF=''
BLASTX="--protein_similarity_search_method diamond_blastx"
SEQUENCES=''
MD5CONF=''
MD5SEQUENCES=''
SEQNAME=''


if [ $# -lt 1 ]; then
  echo -e "\nUsage:\n$0 [--parameters] [--advanced] [--usearch] [--slurm] --sequences <path to fasta file> --config <path to EuGene-P config file (protein and ESTs dbs)>\n"
  echo -e "\nMore details:\n$0 --parameters\n"
  exit 0
fi

PURPLE='\033[0;35m'
NC='\033[0m'
TITLE='\033[1;91m'
PARAMS=`grep -h '=' conf/params/system.config | sed -E 's/[ \t]+//' | grep -v '^/'`
ERROR='\033[0;31m'
INFO='\033[0;32m'

while [ $# -gt 0 ]; do

   if [[ $1 == *"--help"* ]]; then
        echo -e "\nUsage:\n$0 [--parameters]  [--advanced] [--usearch] [--slurm] --sequences <path to fasta file> --config <path to EuGene-P config file (protein and ESTs dbs)>\n"
        exit 0
   elif [[ $1 == *"--parameters"* ]]; then
        nextflow run ./egn-ep.nf $BLASTX --help
        exit 0
   elif [[ $1 == *"--advanced"* ]]; then
        echo -e "${TITLE}[ADVANCED PARAMETERS]${NC}\n\n${PURPLE}${PARAMS}${NC}" | less -r
        exit 0
   elif [[ $1 == *"--config"* ]]; then
        CONF="-c `realpath $2`"
        MD5CONF=`md5sum $2 | cut -d ' ' -f1`
   elif [[ $1 == *"--sequences"* ]]; then
        SEQUENCES=`realpath $2`
        MD5SEQUENCES=`md5sum $2 | cut -d ' ' -f1`
        SEQNAME=`basename $2 | rev | cut -d. -f2- | rev`
   elif [[ $1 == *"--usearch"* ]]; then
        BLASTX="--protein_similarity_search_method ublast_blastx"
   elif [[ $1 == *"--blastx"* ]]; then
        BLASTX="--protein_similarity_search_method blastx"
   elif [[ $1 == *"--slurm"* ]]; then
        SLURM=true
   fi

  shift
done


if [[ $SEQUENCES == '' ]]; then
  echo -e "${ERROR}[ERROR] --sequences parameter is missing${NC}"
  exit 1
elif [[ $CONF == '' ]]; then
  echo -e "${ERROR}[ERROR] --config parameter is missing${NC}"
  exit 2
else
	
  DOTTEST=`grep '^>' $SEQUENCES | cut -f1 -d ' ' | grep '\.'`
  if [[ $DOTTEST != '' ]]; then
	echo -e "${ERROR}[ERROR] Sequence IDs contains '.' character - Please remove them before:\n$DOTTEST${NC}"
	exit 4
  fi
  
  UNIQTEST=`grep '^>' $SEQUENCES | cut -f1 -d ' ' | sort | uniq -c | grep -v ' 1 '`
  if [[ $UNIQTEST != '' ]]; then
	echo -e "${ERROR}[ERROR] Sequence IDs must be unique - Please fix them before:\n$UNIQTEST${NC}"
	exit 5
  fi
  
  RUNDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )/" >/dev/null 2>&1 && realpath . )"
  DATE=`date +%Y%m%d-%s`
  RUNID=`echo "$MD5SEQUENCES" | cut -d ' ' -f1| sed -E 's/\w{24}$//'`
  RESULTID=`echo "$MD5SEQUENCES-$MD5CONF" | cut -d ' ' -f1| sed -E 's/\w{24}$//'`
  LOCKED=`find  $RUNDIR/runs/$SEQNAME-$RUNID/.nextflow/cache/ -name LOCK -exec lsof {} \;`
  
  if [[ $LOCKED == '' ]]; then
     export NXF_SINGULARITY_CACHEDIR=$RUNDIR/runs/$SEQNAME-$RUNID/singularity
     mkdir -p $RUNDIR/runs/$SEQNAME-$RUNID/singularity
     mkdir -p $RUNDIR/runs/$SEQNAME-$RUNID/logs
     rm -f $RUNDIR/runs/$SEQNAME-$RUNID/logs/latest.*
     echo -e "${PURPLE}\n[RUNID]\t >>>>  $SEQNAME-$RUNID  <<<< ${NC}\n\n"
     echo -e "${INFO}[INFO] Launch:  ${NC}\n\tnextflow run $RUNDIR/egn-ep.nf  -resume $ARGS $CONF $BLASTX\n\n"
     echo -e "${INFO}[INFO] Results directory\truns/$SEQNAME-$RUNID/results-$DATE${NC}"
     echo -e "${INFO}[INFO] Logs    directory\truns/$SEQNAME-$RUNID/logs${NC}"
     echo -e "${INFO}[INFO] Work    directory\truns/$SEQNAME-$RUNID/work${NC}"
     echo -e "${INFO}[INFO] Apps    directory\truns/$SEQNAME-$RUNID/singularity${NC}"
     cd $RUNDIR/runs/$SEQNAME-$RUNID/logs/
     ln -s $DATE.nextflow.log latest.nextflow.log
     ln -s $DATE.stdout latest.stdout
     ln -s $DATE.stderr latest.stderr
     cd $RUNDIR/runs/$SEQNAME-$RUNID/
     mkdir results-$DATE
     rm -f latest && ln -s results-$DATE latest
     if [[ ! $SLURM == '' ]]; then
          echo -e "${INFO}[INFO] Submitted to slurm${NC}"
          (cd $RUNDIR/runs/$SEQNAME-$RUNID/ && nextflow \
          -log $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE.nextflow.log \
          run $RUNDIR/egn-ep.nf -ansi-log false -resume $ARGS $CONF $BLASTX \
          -work-dir $RUNDIR/runs/$SEQNAME-$RUNID/work \
          --outdir $RUNDIR/runs/$SEQNAME-$RUNID/results-$DATE \
          > $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE.stdout ) 2> $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE.stderr
     else
          echo -e "${INFO}[INFO] Run in bash mode (nohup and background)${NC}"
          (cd $RUNDIR/runs/$SEQNAME-$RUNID/ && nohup nextflow \
          -log $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE.nextflow.log \
          run $RUNDIR/egn-ep.nf -ansi-log false -resume $ARGS $CONF $BLASTX \
          -work-dir $RUNDIR/runs/$SEQNAME-$RUNID/work \
          --outdir $RUNDIR/runs/$SEQNAME-$RUNID/results-$DATE \
          > $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE.stdout ) 2> $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE.stderr&
     echo -e "${NC}[TIP] Kill Main Process\t\$ ./utils/killRun.sh $SEQNAME-$RUNID${NC}"
     
     fi
     cd $RUNDIR
     echo -e "${NC}[TIP] Follow execution\t\$ tail -f ./runs/$SEQNAME-$RUNID/logs/latest.stdout${NC}"
     echo -e "${NC}[TIP] Failed Tasks\t\$ ./utils/failedTasks.sh $SEQNAME-$RUNID${NC}"
     echo -e "${NC}[TIP] Count Tasks\t\$ ./utils/countTasks.sh $SEQNAME-$RUNID${NC}"
     

  else
     echo -e "${ERROR}[ERROR] There's an already running job for this sequence${NC}"
     echo -e "$LOCKED"
     exit 3
  fi
fi

