nextflow.enable.dsl=2

/*
    &GetBlastXDb($o_param, \%h_database);
    &PrintSection("Protein database cleaning");
    &CleanProtDb($o_param, \%h_database);
    &PrintSection("Protein sequence similarity search");
    &BlastXAllSeq($o_param, $ra_seq, \%h_database, "prot_sim");
*/

process BLASTX_index {
    storeDir "blastIndex/${db.baseName}/"
    label 'process_medium'
    input:
    path (db)

    output:
    path("${db}*")
    
    script:

    makeReduceDbMethodIndex = "diamond makedb --in ${db} -d ${db} && diamond prepdb -d ${db}"
    if (params.protein_similarity_search_method == 'ublast_blastx') {
	    makeReduceDbMethodIndex = "usearch_split_index.pl -splits 8 -cpus ${task.cpus} -in ${db} -outprefix ${db}"
    }

    """
    makeblastdb -dbtype prot -in ${db} -parse_seqids > makeblast.out 2>&1
    ${makeReduceDbMethodIndex}
    """

}

process BLAST_makeblastdb {
    storeDir "blastIndex/${db.baseName}/"
    label 'process_medium'
    
    input:
    path (db)
    val(dbtype)
    val(options)

    output:
    path("${db}*")
    
    script:

    """
    makeblastdb -dbtype ${dbtype} -in ${db} ${options} > makeblast.out 2>&1
    """
}


process BLASTX_indexProtDB {
    tag("${proteinConf.name}")
    
    input:
    tuple val (proteinConf), path(db)

    output:
    tuple val(proteinConf.name), val(proteinConf), path(db), path("${db}.seqlen"), path("${db}*.[apud]*")
    
    script:
    makeReduceDbMethodIndex = "diamond makedb --in ${db} -d ${db} && diamond prepdb -d ${db}"
    if (params.protein_similarity_search_method == 'ublast_blastx') {
        split_number = 8
        makeReduceDbMethodIndex = "usearch_split_index.pl -splits ${split_number} -cpus ${task.cpus} -in ${db} -outprefix ${db}"
    }


    """
    makeblastdb -dbtype prot -in ${db} -parse_seqids > makeblastdb.out 2>&1
    dbprot_extract_seqlen.pl --in ${db} --out ${db}.seqlen
    ${makeReduceDbMethodIndex}
    """

}

process BLASTX_cleanProtDB {
    label 'process_medium'
    tag("${proteinConf.name}")
    
    input:
    tuple val (proteinConf), path(db)
    val(repeatConf)

    output:
    tuple val(proteinConf.name), val(proteinConf), path(cleandb), path("${cleandb}.seqlen"), path("${cleandb}*.[apud]*")
    
    script:

    cleandb = db.baseName + ".noTE"
    makeReduceDbMethodIndex = "diamond makedb --in ${cleandb} -d ${cleandb} && diamond prepdb -d ${cleandb}"
    if (params.protein_similarity_search_method == 'ublast_blastx') {
        split_number = 8
        makeReduceDbMethodIndex = "usearch_split_index.pl -splits ${split_number} -cpus ${task.cpus} -in ${cleandb} -outprefix ${cleandb}"
    }

    blastparam = ""
    if (params.prg_remove_repbase_blastparam ne '') {
        blastparam = "--blastparam ${params.prg_remove_repbase_blastparam} "
    } 
    repeat_sequence_db = repeatConf['repeat_sequence_db']
    
    """
    lipm_dbprot_remove_repbase.pl --in ${db} --out ${cleandb} --repbase ${repeat_sequence_db} ${blastparam} --blastpprg blastp --fastafilterprg  lipm_fastafilter.pl
    makeblastdb -dbtype prot -in ${cleandb} -parse_seqids > makeblast.out 2>&1
    dbprot_extract_seqlen.pl --in ${cleandb} --out ${cleandb}.seqlen
    ${makeReduceDbMethodIndex}
    """

    
    
}

process BLASTX_alignHigh {
    label 'process_high'
     tag("${sequence.baseName} - ${dbname}")

    input:
    tuple  val(name), val (proteinConf), path(db), path(seqlen), path(index), path(sequence)
    
    output:
    tuple  val (seqid), val(proteinConf.name), val (proteinConf),path("${sequence}.${proteinConf.name}.m8")

    script:
    
    dbname = db.baseName
    if (db.extension != '')  {
        dbname =  db.baseName + '.' +  db.extension
    }

    seqid = sequence.baseName
    if (sequence.extension != '')  {
        seqid =  sequence.baseName + '.' +  sequence.extension
    }

    search_method_param = " --prg_diamond_param '${params.prg_diamond_param} --max-target-seqs ${proteinConf.entries}'"
    splitdb = ''
    if (params.protein_similarity_search_method == 'ublast_blastx') {
        search_method_param = " --prg_usearch_ublast_param '${params.prg_usearch_ublast_param}'"
        if ( params.split_blastx_db) {
        splitdb = ' --splitdb '
        }
    }

    fil_cmd = "awk '{ if (\$3 >= ${proteinConf.pci}) print }'  ${sequence}.${proteinConf.name}.m8.raw > ${sequence}.${proteinConf.name}.m8"
    
    """
    targeted_blast_search.pl --cpus ${task.cpus} --query  ${sequence} \
     ${splitdb} --db ${dbname} \
     --blast_param '${params.blastx_param}' \
     --output \$PWD/${sequence}.${proteinConf.name}.m8.raw \
     --protein_similarity_search_method ${params.protein_similarity_search_method} ${params.prg_targetedblastsearch_param} ${search_method_param}
     ${fil_cmd}
    [ -s ${sequence}.${proteinConf.name}.m8 ] && touch  ${sequence}.${proteinConf.name}.m8 ||  echo "#noresult" > ${sequence}.${proteinConf.name}.m8
    """


}

process BLASTX_alignMedium {
    label 'process_medium'
    tag("${sequence.baseName} - ${dbname}")

    input:
    tuple  val(name), val (proteinConf), path(db), path(seqlen), path(index), path(sequence)
    
    output:
    tuple  val (seqid), val(proteinConf.name), val (proteinConf),  path("${sequence}.${proteinConf.name}.m8")


    script:
    
    dbname = db.baseName
    if (db.extension != '')  {
        dbname =  db.baseName + '.' +  db.extension
    }

    seqid = sequence.baseName
    if (sequence.extension != '')  {
        seqid =  sequence.baseName + '.' +  sequence.extension
    }

    search_method_param = " --prg_diamond_param '${params.prg_diamond_param} --max-target-seqs ${proteinConf.entries}'"
    splitdb = ''
    if (params.protein_similarity_search_method == 'ublast_blastx') {
        search_method_param = " --prg_usearch_ublast_param '${params.prg_usearch_ublast_param}'"
        if ( params.split_blastx_db) {
            splitdb = ' --splitdb '
        }
    }

    fil_cmd = "awk '{ if (\$3 >= ${proteinConf.pci}) print }'  ${sequence}.${proteinConf.name}.m8.raw > ${sequence}.${proteinConf.name}.m8"
    
    """
    targeted_blast_search.pl --cpus ${task.cpus} --query  ${sequence} \
     ${splitdb} --db ${dbname} \
     --blast_param '${params.blastx_param}' \
     --output \$PWD/${sequence}.${proteinConf.name}.m8.raw \
     --protein_similarity_search_method ${params.protein_similarity_search_method} ${params.prg_targetedblastsearch_param} ${search_method_param}
     ${fil_cmd}
    [ -s ${sequence}.${proteinConf.name}.m8 ] && touch  ${sequence}.${proteinConf.name}.m8 ||  echo "#noresult" > ${sequence}.${proteinConf.name}.m8
    """

}

process BLASTX_alignLow {
    label 'process_low'
    tag("${sequence.baseName} - ${dbname}")

    input:
    tuple  val(name), val (proteinConf), path(db), path(seqlen), path(index), path(sequence)
    
    output:
    tuple  val (seqid), val(proteinConf.name), val (proteinConf), path("${sequence}.${proteinConf.name}.m8")


    script:
    
    dbname = db.baseName
    if (db.extension != '')  {
        dbname =  db.baseName + '.' +  db.extension
    }

    seqid = sequence.baseName
    if (sequence.extension != '')  {
        seqid =  sequence.baseName + '.' +  sequence.extension
    }

    search_method_param = " --prg_diamond_param '${params.prg_diamond_param} --max-target-seqs ${proteinConf.entries}'"
    splitdb = ''
    if (params.protein_similarity_search_method == 'ublast_blastx') {
        search_method_param = " --prg_usearch_ublast_param '${params.prg_usearch_ublast_param}'"
        if ( params.split_blastx_db) {
            splitdb = ' --splitdb '
        }
    }
    
    fil_cmd = "awk '{ if (\$3 >= ${proteinConf.pci}) print }'  ${sequence}.${proteinConf.name}.m8.raw > ${sequence}.${proteinConf.name}.m8"
    
    """
    targeted_blast_search.pl --cpus ${task.cpus} --query  ${sequence} \
     ${splitdb} --db ${dbname} \
     --blast_param '${params.blastx_param}' \
     --output \$PWD/${sequence}.${proteinConf.name}.m8.raw \
     --protein_similarity_search_method ${params.protein_similarity_search_method} ${params.prg_targetedblastsearch_param} ${search_method_param}
     ${fil_cmd}
    [ -s ${sequence}.${proteinConf.name}.m8 ] && touch  ${sequence}.${proteinConf.name}.m8 ||  echo "#noresult" > ${sequence}.${proteinConf.name}.m8
    """

}


process BLASTX_Repeats {
    label 'process_high'
    tag("${seqid} - blastx repeat")

    input:
    tuple  val(seqid),  path(sequence)
    path(repeatFasta)
    path(repeatFastaIndex)
    
    output:
    tuple  val (seqid), val('repeat'), val (conf), path("${seqid}.repeat.m8")

    script:

    conf = [:]
    conf['name'] = 'repeat'
    search_method_param = " --prg_diamond_param '${params.prg_diamond_param} --max-target-seqs 1000000000'"
    splitdb = ''
    if (params.protein_similarity_search_method == 'ublast_blastx') {
        search_method_param = " --prg_usearch_ublast_param '${params.prg_usearch_ublast_param}'"
        if ( params.split_blastx_db) {
            splitdb = ' --splitdb '
        }
    }

    """
    targeted_blast_search.pl --cpus ${task.cpus} --query  ${sequence} \
     ${splitdb} --db ${repeatFasta} \
     --blast_param '${params.blastx_param}' \
     --output \$PWD/${seqid}.repeat.m8 \
     --protein_similarity_search_method ${params.protein_similarity_search_method} ${params.prg_targetedblastsearch_param} ${search_method_param}
    [ -s ${seqid}.repeat.m8 ] && touch ${seqid}.repeat.m8 ||  echo "#noresult" > ${seqid}.repeat.m8
    """
}


process BLASTX_postProcessing {
    
    tag("${seqid} - ${proteinConf.name}")

    input:
    tuple val (proteinConf),val (seqid), path (gff3file)

    output:
    tuple val (seqid), path ("${outprefix}*.gff3")
    
    script:
    seqlen = proteinConf.seqlen
    outprefix = "${seqid}.${proteinConf.name}"

    independent_strand_annotation = "touch ${outprefix}.gff3"
    if (params.independent_strand_annotation) {
        independent_strand_annotation = "egn_splitbystrand.pl --gff3 ${outprefix}.gff3"
    }

    cmd = "cp ${gff3file} ${outprefix}.gff3"
    unique = ""
    if (params.blastx_unique_filter) {
        unique = " --unique "
    }
    if (proteinConf.containsKey("pcs") && proteinConf.pcs != '') {
        cmd = "filter_protein_similarities.pl ${unique} --seqlen ${seqlen} --scov ${proteinConf.pcs} --in ${gff3file} --outfile ${outprefix}.gff3"
    }


    """
    ${cmd}
    [ -s ${outprefix}.gff3 ] && touch ${outprefix}.gff3 ||  echo "##gff-version 3" > ${outprefix}.gff3
    ${independent_strand_annotation}
    """
}

process BLASTX_alignForTraining {
    
    label 'process_medium'
    errorStrategy { task.exitStatus in 0..1 ? 'ignore' : 'terminate' }
    
    input:
    path(query)
    path(db)
    path(indices)
    
    output:
    path ("${query}.${db.baseName}.blasttraining.gz")

    script:
    """
    blastx -db ${db} -query ${query} -num_threads ${task.cpus} ${params.hsp_training_blastx_param} | gzip -c > ${query}.${db.baseName}.blasttraining.gz
    """
}


process BLAST_extractRepeatDomain {
    label "process_high"
    errorStrategy { task.exitStatus in 0..1 ? 'ignore' : 'terminate' }

    input:
    path(protv0)
    val(repeatConf)
    path(indices)

    output:
    path("${protv0}.repeat")
    path("${protv0}.All_SpeciesRepeatDomain.fa")

    script:

    blast_param = ''
    dbtype = 'prot'
    blast_prg = 'blastp'

    if ( params.prg_extract_repeat_domain_blastparam ) { blast_param = " --blastparam ${params.prg_extract_repeat_domain_blastparam}" }
    
    if (params.use_repbase)
    {
        if (! repeatConf['repeat_sequence_db'] ) { exit 1, 'repeat_sequence_db path not specified!' }
        repeat_sequence_db = repeatConf['repeat_sequence_db']
        repeat_db_name = repeat_sequence_db.split("/").last()

        """
        if [ -s ${protv0} ]
        then
			ln -s ${repeat_sequence_db} ${repeat_db_name}
			dbprot_extract_repeat_domain.pl --blast_prg ${blast_prg} ${blast_param} --in ${protv0} --out ${protv0}.repeat --repbase ${repeat_db_name} --scov ${params.prg_extract_repeat_domain_min_scov} --cpus ${task.cpus}
			cat ${protv0}.repeat ${repeat_sequence_db} >  ${protv0}.All_SpeciesRepeatDomain.fa
		else
			touch ${protv0}.repeat
			touch ${protv0}.All_SpeciesRepeatDomain.fa
		fi
        """
    }
    else 
    {

        if (! repeatConf['dfam_consensus_db'] ) { exit 1, 'dfam_consensus_db path not specified!' }
        dfam_consensus_db = repeatConf['dfam_consensus_db']
        if (! repeatConf['transposonpsi_db'] ) { exit 1, 'transposonpsi_db path not specified!' }
        transposonpsi_db = repeatConf['transposonpsi_db'] 
        
        blast_prg = 'tblastn'
        dbtype = 'nucl'
        repeat_db_name = dfam_consensus_db.split("/").last()
        transposonpsi_db_name = transposonpsi_db.split("/").last()

        """
        if [ -s ${protv0} ]
        then
			ln -s ${dfam_consensus_db} ${repeat_db_name}
			ln -s ${transposonpsi_db} ${transposonpsi_db_name}
			dbprot_extract_repeat_domain.pl --blast_prg ${blast_prg} ${blast_param} --in ${protv0} --out ${protv0}.dfam.repeat --repbase ${repeat_db_name} --scov ${params.prg_extract_repeat_domain_min_scov} --cpus ${task.cpus}
			dbprot_extract_repeat_domain.pl --blast_prg blastp ${blast_param} --in ${protv0} --out ${protv0}.transposonpsi.repeat --repbase ${transposonpsi_db_name} --scov ${params.prg_extract_repeat_domain_min_scov} --cpus ${task.cpus}
			cat ${protv0}.dfam.repeat ${protv0}.transposonpsi.repeat > ${protv0}.repeat
			cat ${protv0}.repeat  ${transposonpsi_db_name} > ${protv0}.All_SpeciesRepeatDomain.fa
		else
			touch ${protv0}.repeat
			touch ${protv0}.All_SpeciesRepeatDomain.fa
		fi
        """
    }
    
}

process BLAST_softwaresInfo {
	output:
	path("blast.softwares_info.tsv")

	script:
	"""
	softwares_info > blast.softwares_info.tsv
	"""
}
